=== modified file 'drizzled/log.cc'
--- drizzled/log.cc	2008-12-05 19:17:37 +0000
+++ drizzled/log.cc	2008-12-06 06:21:20 +0000
@@ -27,6 +27,7 @@
 #include <drizzled/server_includes.h>
 #include <drizzled/replication/replication.h>
 #include <libdrizzle/libdrizzle.h>
+#include <drizzled/replicator.h>
 #include <mysys/hash.h>
 #include <drizzled/replication/rli.h>
 
@@ -487,6 +488,8 @@
   {
     Query_log_event qev(session, STRING_WITH_LEN("COMMIT"), true, false);
     qev.error_code= 0; // see comment in DRIZZLE_LOG::write(Session, IO_CACHE)
+    /* TODO: Fix return type */
+    (void)replicator_end_transaction(session, all, true);
     int error= binlog_end_trans(session, trx_data, &qev, all);
     return(error);
   }
@@ -545,6 +548,10 @@
      */
     error= binlog_end_trans(session, trx_data, 0, all);
   }
+
+  /* TODO: Fix return type */
+  (void)replicator_end_transaction(session, all, false);
+
   return(error);
 }
 
@@ -3618,6 +3625,8 @@
   Xid_log_event xle(session, xid);
   binlog_trx_data *trx_data=
     (binlog_trx_data*) session_get_ha_data(session, binlog_hton);
+  /* TODO: Fix return type */
+  (void)replicator_end_transaction(session, true, true);
   /*
     We always commit the entire transaction when writing an XID. Also
     note that the return value is inverted.

=== modified file 'drizzled/plugin_replicator.h'
--- drizzled/plugin_replicator.h	2008-12-06 02:02:19 +0000
+++ drizzled/plugin_replicator.h	2008-12-06 06:21:20 +0000
@@ -39,6 +39,7 @@
                      const unsigned char *before, 
                      const unsigned char *after);
   bool (*row_delete)(Session *session, Table *table);
+  bool (*end_transaction)(Session *session, bool autocommit, bool commit);
 } replicator_t;
 
 #endif /* DRIZZLED_PLUGIN_REPLICATOR_H */

=== modified file 'drizzled/replicator.cc'
--- drizzled/replicator.cc	2008-12-06 02:02:19 +0000
+++ drizzled/replicator.cc	2008-12-06 06:21:20 +0000
@@ -235,3 +235,51 @@
   return replicator_do_row(session, &param);
 }
 
+/* 
+  Here be Dragons!
+
+  Ok, not so much dragons, but this is where we handle either commits or rollbacks of 
+  statements. 
+*/
+typedef struct replicator_row_end_st
+{
+  bool autocommit;
+  bool commit;
+} replicator_row_end_st;
+
+/* We call this to end a statement (on each registered plugin) */
+static bool replicator_do_row_end_iterate (Session *session, plugin_ref plugin, void *p)
+{
+  replicator_t *repl= plugin_data(plugin, replicator_t *);
+  replicator_row_end_st *params= (replicator_row_end_st *)p;
+
+  /* call this loaded replicator plugin's replicator_func1 function pointer */
+  if (repl && repl->end_transaction)
+  {
+    if (repl->end_transaction(session, params->autocommit, params->commit))
+    {
+      /* TRANSLATORS: The leading word "replicator" is the name
+        of the plugin api, and so should not be translated. */
+      sql_print_error(_("replicator plugin '%s' end_transaction() failed"),
+                      (char *)plugin_name(plugin));
+      return true;
+    }
+  }
+
+  return false;
+}
+
+bool replicator_end_transaction(Session *session, bool autocommit, bool commit)
+{
+  bool foreach_rv;
+  replicator_row_end_st params;
+  
+  params.autocommit= autocommit;
+  params.commit= commit;
+
+  /* We need to free any data we did an init of for the session */
+  foreach_rv= plugin_foreach(session, replicator_do_row_end_iterate,
+                             DRIZZLE_REPLICATOR_PLUGIN, (void *) &params);
+
+  return foreach_rv;
+}

=== modified file 'drizzled/replicator.h'
--- drizzled/replicator.h	2008-12-06 02:02:19 +0000
+++ drizzled/replicator.h	2008-12-06 06:21:20 +0000
@@ -34,5 +34,6 @@
                            const unsigned char *before, 
                            const unsigned char *after);
 bool replicator_delete_row(Session *session, Table *table);
+bool replicator_end_transaction(Session *session, bool autocommit, bool commit);
 
 #endif /* DRIZZLED_REPLICATOR_H */

